clear;
randnum = 20;
methods = {'LRBFGS', 'LRTRSR1', 'WFlow'}; % {'WFlow'}; % {'LRBFGS', 'LRTRSR1'}; % 
n1s = [16, 32, 64, 128, 256, 512];
n2s = [16, 32, 64, 128, 256, 512];
ls = [8, 8, 8, 8, 8, 8];

prob = 1;
tab = zeros(18, 6, randnum);
for i = 1 : randnum
    seed = i;
    for j = 1 : length(methods)
        for k = 1 : length(n1s)
            [i, j, k]
            [Xopt, f, gf, gfgf0, iter, nf, ng, nR, nV, nVp, nH, ComTime, funs, grads, times] = Testproblem(seed, methods{j}, prob, n1s(k), n2s(k), ls(k));
            tab(1 + j * 6 - 6, k, i) = iter;
            tab(2 + j * 6 - 6, k, i) = nf;
            tab(3 + j * 6 - 6, k, i) = ng;
            tab(4 + j * 6 - 6, k, i) = gfgf0;
            tab(5 + j * 6 - 6, k, i) = f;
            tab(6 + j * 6 - 6, k, i) = ComTime;
        end
    end
end

TimeTab(:, 1) = reshape(tab(6, :, :), [], 1);
TimeTab(:, 2) = reshape(tab(12, :, :), [], 1);
TimeTab(:, 3) = reshape(tab(18, :, :), [], 1);

minTime = min(TimeTab')';
minTime = repmat(minTime, 1, 3);
perfratio = TimeTab ./ minTime;

taumax = max(perfratio(:));

figure(1);clf
set(0,'defaultaxesfontsize',15, ...
   'defaultaxeslinewidth',0.7, ...
   'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
set(0,'defaultlinemarkersize',10)
shapes = {'-b', '--k', '-.m', ':r', 'm', 'y'};
for i = 1 : 3
    v = sort(perfratio(:, i));
    xy = [v, ((1:length(v)) / length(v))'];
    xy = [1, 0; xy; taumax, 1];
    xynew = zeros(2 * size(xy, 1) - 1, 2);
    xynew(1:2:end, :) = xy;
    for j = 2 : 2 : size(xynew, 1)
        xynew(j, 1) = xynew(j + 1, 1);
        xynew(j, 2) = xynew(j - 1, 2);
    end

    plot(xynew(:, 1), xynew(:, 2), [shapes{i}]); hold on;
end
legend('LRBFGS', 'LRTRSR1-R', 'Wirtinger flow');
xlabel('\tau')
ylabel('\rho_s(\tau)');
title('Performance profile on phase retrieval');
savefig('CSFOPhaseRetLRBFGSvsLRTRSR1_fig');

avetab = mean(tab, 3);

fout = fopen('CSFOPhaseRetLRBFGSvsLRTRSR1.txt','w');
for i = 1 : size(avetab, 1)
    for j = 1 : size(avetab, 2)
        if(mod(i, 6) == 1 || mod(i, 6) == 2 || mod(i, 6) == 3)
            fprintf(fout, '%d', round(avetab(i, j)));
        elseif(mod(i, 6) == 4 || mod(i, 6) == 5 || mod(i, 6) == 0)
            fprintf(fout, '$%s$ ', outputfloat(avetab(i, j)));
        end
        if(j < size(avetab, 2))
            fprintf(fout, ' & ', avetab(i, j));
        end
    end
    fprintf(fout,'\\\\\n');
end
fclose(fout);

function [Xopt, f, gf, gfgf0, iter, nf, ng, nR, nV, nVp, nH, ComTime, funs, grads, times] = Testproblem(seed, method, prob, n1, n2, l)
    rng(seed);
    if(prob == 1)
        fprintf('MTestCSFRQPhaseRetrieval seed:%d\n', seed);
        
        n = n1 * n2;
        m = l * n;
        k = 1;
        kappa = 0; %1 / sqrt(n);

        %% generate masks and a true solution.
        masks = complex(randn(n1, n2, l), randn(n1, n2, l));
        phase = complex(randn(n1, n2), randn(n1, n2));
        phase = phase / norm(phase);
        b = zeros(m, 1);
        for i = 1 : l
            zi = reshape(fft2(phase .* masks(:, :, i)) / sqrt(n1 * n2), [], 1);
            b((i - 1) * n + 1 : i * n) = conj(zi) .* zi;
        end
        %%
        tic
        tmp = conj(masks).* masks;
        lambda = sqrt(sum(b) / real(sum(tmp(:))) * n);
        x1.U = randn(n, k);
        [x1.U, ~] = qr(x1.U, 0);
        for i = 1 : 10
            x1.U = Ax(x1.U, masks, b, kappa);
            [x1.U, ~] = qr(x1.U, 0);
        end
        x1.U = lambda * x1.U;
        toc

        initX = x1.U;
        masksn = masks;

        if(strcmp(method, 'WFlow'))
            isRmethod = 0;
            SolverParams.method = method;
            
            if(n1 == 16 && n2 == 16)
                k0 = 330; mumax = 0.04; mu0 = 50;
            elseif(n1 == 32 && n2 == 32)
                k0 = 330; mumax = 0.13; mu0 = 100;
            elseif(n1 == 64 && n2 == 64)
                k0 = 330; mumax = 0.25; mu0 = 200;
            elseif(n1 == 128 && n2 == 128)
                k0 = 330; mumax = 0.40; mu0 = 400;
            elseif(n1 == 256 && n2 == 256)
                k0 = 330; mumax = 0.45; mu0 = 1600;
            elseif(n1 == 512 && n2 == 512)
                k0 = 330; mumax = 0.48; mu0 = 6400;
            else
                error('not tune yet\n');
            end
            
            WFmaxiter = 2000;
            ParamSet = 2;
            HasHHR = 0;
            [Xopt, ComTime, nfft, iter, gfgf0, f] = TestCSFRQPhaseRetrieval(b, masksn, initX, kappa, ParamSet, HasHHR, isRmethod, SolverParams, k0, mumax, mu0, WFmaxiter);
            gf = 0; nf = 0; ng = iter + 1; nR = 0; nV = 0; nVp = 0; nH = 0; funs = 0; grads = 0; times = 0;
            fprintf('final f:%e, time:%g, nfft:%d\n', f, ComTime, nfft);
            return;
        end
        
        SolverParams.method = method;
        SolverParams.IsCheckParams = 1;
        SolverParams.Max_Iteration = 1000;
        SolverParams.LengthSY = 4;
        SolverParams.Verbose = 2;
%         SolverParams.LMrestart = 0;
%         SolverParams.Accuracy = 1e-6;
%         SolverParams.Tolerance = 1e-9;
        SolverParams.OutputGap = 100;
        SolverParams.Finalstepsize = 1;
        SolverParams.Num_pre_funs = 1;
        SolverParams.PreFunsAccuracy = 1e6;
        HasHHR = 0;
        ParamSet = 2;
        isRmethod = 1;
        [Xoptt, f, gf, gfgf0, iter, nf, ng, nR, nV, nVp, nH, ComTime, funs, grads, times] = TestCSFRQPhaseRetrieval(b, masksn, initX, kappa, ParamSet, HasHHR, isRmethod, SolverParams);
        Xopt = Xoptt.main;
    end 
end

function str = outputfloat(x)
    if(x <= 0)
        sn = '-';
        x = abs(x);
    else
        sn = '';
    end
    p = log(x)/log(10);
    p = - ceil(-p);
    x = round(x * 10^(-p) * 100);
    x = x / 100;
    strx = sprintf('%3.2f', x);
    if(p ~= 0)
        str = [sn strx '_{' num2str(p) '}'];
    else
        str = [sn strx];
    end
end

function output = Ax(U, masks, b, kappa)
    x.U = U;
    [n1, n2, l] = size(masks);
    n = n1 * n2;
    m = l * n;
    k = size(x.U, 2);
    ZY = zeros(m, k);
    for i = 1 : l
        for j = 1 : k
            temp = fft2(reshape(x.U(:, j), n1, n2) .* masks(:, :, i)) / sqrt(n1 * n2);
            ZY((i - 1) * n + 1 : i * n, j) = reshape(temp, [], 1);
        end
    end
    DZY = zeros(m, k);
    for i = 1 : k
        DZY(:, i) = b .* ZY(:, i);
    end
    %% Z^H *DZY
    Y = zeros(n, k);
    for i = 1 : l
        temp = zeros(n, k);
        for j = 1 : k
            temp(:, j) = reshape(ifft2(reshape(DZY((i - 1) * n + 1 : i * n, j) * sqrt(n1 * n2), n1, n2)), [], 1);
            temp(:, j) = conj(reshape(masks(:, :, i), [], 1)) .* temp(:, j);
        end
        Y = Y + temp;
    end
    output = Y;
end
% 
% function output = Afun(x, tflag, A)
%     if strcmp(tflag,'notransp')
%         output = A * x;
%     else
%         output = A' * x;
%     end
% end
% 
% function output = computeAH(As, Type)
% 
%     k = length(As);
%     
%     for i = 1:k
%         invAs{i} = pinv(As{i});
%     end
%     
%     Ar = As{1};
%     for i = 2:k
%        Ar = Ar + As{i};
%     end
%     Ar = Ar/k;
%     
%     if (Type == 0)
%         output = Ar;
%     end
%     
%     
%     if(Type == 1)
%     Ha = pinv(As{1});
%     for i = 2:k
%         Ha = Ha + invAs{i};
%     end
%     
%     Ha = pinv(Ha/k);
%     Arh = Ar^(0.5);
%     Arinvh = Ar^(-0.5);
%     temp = Arinvh * Ha * Arinvh;
%     output = Arh * temp^(0.5) * Arh;
%     end
% end
