
randnum = 20;
methods = {'LRBFGS', 'LRTRSR1', 'RCG', 'LMAFit'}; %  {'LRBFGS'}; % 
prob = 1;
ns = [2000, 2000, 2000, 4000, 4000, 4000, 8000, 8000, 8000];
rs = [10, 20, 30, 10, 20, 30, 10, 20, 30];
tab = zeros(16, 9, randnum);
for i = 1 : randnum
    seed = i;
    for j = 1 : length(methods)
        for k = 1 : length(ns)
            [i, j, k]
            [Xopt, f, gf, gfgf0, iter, nf, ng, nR, nV, nVp, nH, ComTime, funs, grads, times, fail] = Testproblem(seed, methods{j}, prob, ns(k), rs(k));
            tab(1 + j * 4 - 4, k, i) = iter;
            tab(2 + j * 4 - 4, k, i) = nf;
            tab(3 + j * 4 - 4, k, i) = f;
            tab(4 + j * 4 - 4, k, i) = ComTime;
        end
    end
end
TimeTab = [];
TimeTab(:, 1) = reshape(tab(4, :, :), [], 1);
TimeTab(:, 2) = reshape(tab(8, :, :), [], 1);
TimeTab(:, 3) = reshape(tab(12, :, :), [], 1);
TimeTab(:, 4) = reshape(tab(16, :, :), [], 1);

minTime = min(TimeTab')';
minTime = repmat(minTime, 1, 4);
perfratio = TimeTab ./ minTime;

taumax = max(perfratio(:));

figure(1);clf
set(0,'defaultaxesfontsize',15, ...
   'defaultaxeslinewidth',0.7, ...
   'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
set(0,'defaultlinemarkersize',10)
shapes = {'-b', '--k', '-.m', ':r', 'm', 'y'};
for i = 1 : 4
    v = sort(perfratio(:, i));
    xy = [v, ((1:length(v)) / length(v))'];
    xy = [1, 0; xy; taumax, 1];
    xynew = zeros(2 * size(xy, 1) - 1, 2);
    xynew(1:2:end, :) = xy;
    for j = 2 : 2 : size(xynew, 1)
        xynew(j, 1) = xynew(j + 1, 1);
        xynew(j, 2) = xynew(j - 1, 2);
    end

    plot(xynew(:, 1), xynew(:, 2), [shapes{i}]); hold on;
end
legend('LRBFGS', 'LRTRSR1-R', 'RCGR', 'LMAFit');
xlabel('\tau')
ylabel('\rho_s(\tau)');
title('Performance profile on matrix completion');
savefig('FRankE3MatComLRBFGSvsLRTRSR1_fig');

avetab = mean(tab, 3);

fout = fopen('FRankE3MatComLRBFGSvsLRTRSR1.txt','w');
for i = 1 : size(avetab, 1)
    for j = 1 : size(avetab, 2)
        if(mod(i, 4) == 1 || mod(i, 4) == 2)
            fprintf(fout, '%d ', round(avetab(i, j)));
        elseif(mod(i, 4) == 3 || mod(i, 4) == 0)
            fprintf(fout, '$%s$ ', outputfloat(avetab(i, j)));
        end
        if(j < size(avetab, 2))
            fprintf(fout, ' & ', avetab(i, j));
        end
    end
    fprintf(fout,'\\\\\n');
end
fclose(fout);

function [Xopt, f, gf, gfgf0, iter, nf, ng, nR, nV, nVp, nH, ComTime, funs, grads, times, fail] = Testproblem(seed, method, prob, inn, inr)
    rng(seed);
    fail = 0;
 
    if(prob == 1)
        fprintf('LRMatrixcompletion seed:%d\n', seed);
        
        
        m = inn;  % row of the matrix
        n = inn;  % column of the matrix
        r = inr;    % rank of the matrix
%         m = 8;  % row of the matrix
%         n = 7;  % column of the matrix
%         r = 2;    % rank of the matrix
        OS = 3;
        Tolerance = 1e-10; % tolerance for relative residual

        % initial iterate
        Xinit = randn(m, r);
        Yinit = randn(n, r);
        [Qx, Rx] = qr(Xinit, 0);
        [Qy, Ry] = qr(Yinit, 0);
        U = Qx;
        V = Qy;
        D = Rx * Ry';
%         U = randn(m, r);
%         D = randn(r, r);
%         V = randn(n, r);
        
        sigma = 0e-2;
        Xs = randn(m,r);
        Ys = randn(r,n);
        L = OS * (m + n - r) * r;

        Known = unique(round(rand(1,2*L)*m*n));
        if Known(1) == 0; Known = Known(2:end); end
        Lc = length(Known);
        if Lc > L
            p = randperm(Lc);
            Known = sort(Known(p(1:L)));
        else
            L = Lc;
        end
        OStrue = L / ((m + n - r) * r)

        % generate data
        [Ik, Jk] = ind2sub([m n],Known);
        data = partXY(Xs',Ys,Ik,Jk,L);

        % add noise
        noise = randn(size(data));
        level = sigma*norm(data)/norm(noise);
        data = data + level*noise; 
        A = sparse(Ik, Jk, data);

%         tic; A * Xs; toc
%         tic; Xs' * A; toc
%         error('sdf');
        
%         tic
%         [U, D, V] = svds(@(x, tflag)Afun(x, tflag, A), [m, n], r);
%         toc

        if(strcmp(method, 'LMAFit')) % if use LMaFit method
            % % error calculation functions
            lrdot = @(X1,Y1,X2,Y2) sum(sum((X1'*X2).*(Y1*Y2')));
            % problem specification
            opts = [];
            opts.tol = Tolerance;
            opts.maxit = 2000;
            %opts.Zfull = 0;
            %opts.DoQR  = 1;
            opts.print = 0;
            opts.est_rank = 0; % rank is fixed
            opts.init = 1;
            opts.X = U * D;
            opts.Y = V';

            % call solver
            tic; 
            [X,Y,Out] = lmafit_mc_adp(m,n,r,Known,data,opts); 
            ComTime = toc;
            Xopt = []; gf = []; gfgf0 = []; ng = []; nH = []; nR = []; nV = []; nVp = []; grads = []; times = [];
            f = Out.obj(end);
            iter = Out.iter;
            nf = length(Out.obj);
            funs = Out.obj;
            
            fprintf('i:%d,f0:%.3e,f:%.3e,nf:%d,time:%f\n', iter,(funs(1) * norm(data))^2/2,(f * norm(data))^2/2,nf,ComTime);
            return;
        end
        
%         m = 4000;
%         n = 4000;
%         r = 40;
        
%         G = randn(m, r);
%         H = randn(n, r);
%     %     B = G * H';
%         OS = 3;
%     %     [(m + n - r) * r * OS, m * n]
%         nz = min((m + n - r) * r * OS, m * n);
%         vidx = randperm(m * n, nz);
%         [ir, jc] = ind2sub([m, n], vidx);
%         Bv = zeros(1, length(ir));
%         for i = 1 : length(Bv)
%             Bv(i) = G(ir(i), :) * H(jc(i), :)';
%         end
%     %     norm(Bv - B(vidx))
%         A = sparse(ir, jc, Bv);
    
%         tic
%         [U, D, V] = svds(@(x, tflag)Afun(x, tflag, A), [m, n], r);
%         toc
%         fprintf('Get leading singular vectors of A\n');
        Xinitial = [U(:); D(:); V(:)];
        SolverParams.method = method;
        SolverParams.IsCheckParams = 1;
        SolverParams.Max_Iteration = 2000;
        SolverParams.OutputGap = 2000;
%         SolverParams.LengthSY = 1;
        SolverParams.Num_pre_funs = 10;
        SolverParams.Accuracy = 1e-10;
        SolverParams.verbose = 1;
        SolverParams.Stop_Criterion = 3;
%         SolverParams.LMrestart = 1;
        SolverParams.Tolerance = (Tolerance * norm(data))^2 / 2;
        if(strcmp(method, 'RCG')) % if it is RCG method
            SolverParams.LineSearch_LS = 4;
            SolverParams.RCGmethod = 1;
        end
        HasHHR = 0;
        [Xopt, f, gf, gfgf0, iter, nf, ng, nR, nV, nVp, nH, ComTime, funs, grads, times] = ...
            TestFRankE3FMatCompletion(A, Xinitial, r, HasHHR, SolverParams);
%         error('sf');%%----
        funs = sqrt(funs * 2) / norm(data);
        f = funs(end);
        if(f > 1e-1)
            fail = 1;
        end
    end
end

function str = outputfloat(x)
    if(x <= 0)
        sn = '-';
        x = abs(x);
    else
        sn = '';
    end
    p = log(x)/log(10);
    p = - ceil(-p);
    x = round(x * 10^(-p) * 100);
    x = x / 100;
    strx = sprintf('%3.2f', x);
    if(p ~= 0)
        str = [sn strx '_{' num2str(p) '}'];
    else
        str = [sn strx];
    end
end

function output = Afun(x, tflag, A)
    if strcmp(tflag,'notransp')
        output = A * x;
    else
        output = A' * x;
    end
end

function output = computeAH(As, Type)

    k = length(As);
    
    for i = 1:k
        invAs{i} = pinv(As{i});
    end
    
    Ar = As{1};
    for i = 2:k
       Ar = Ar + As{i};
    end
    Ar = Ar/k;
    
    if (Type == 0)
        output = Ar;
    end
    
    if(Type == 1)
    Ha = pinv(As{1});
    for i = 2:k
        Ha = Ha + invAs{i};
    end
    
    Ha = pinv(Ha/k);
    Arh = Ar^(0.5);
    Arinvh = Ar^(-0.5);
    temp = Arinvh * Ha * Arinvh;
    output = Arh * temp^(0.5) * Arh;
    end
end
