clear;
randnum = 20;
methods = {'LRBFGS', 'LRTRSR1', 'RTRNewton'};
ns = [48, 48, 48, 48, 48, 48, 96, 96, 96];
ps = [16, 16, 16, 32, 32, 32, 32, 32, 32];
Ns = [1000, 2000, 4000, 1000, 2000, 4000, 1000, 2000, 4000];

prob = 1;
tab = zeros(21, 9, randnum);
for i = 1 : randnum
    failout = 0;
    seed = i;
    for j = 1 : length(methods)
        for k = 1 : length(ns)
            [i, j, k]
            [Xopt, f, gf, gfgf0, iter, nf, ng, nR, nV, nVp, nH, ComTime, funs, grads, times, fail] = Testproblem(seed, methods{j}, prob, ns(k), ps(k), Ns(k));
            tab(1 + j * 7 - 7, k, i) = iter;
            tab(2 + j * 7 - 7, k, i) = nf;
            tab(3 + j * 7 - 7, k, i) = ng;
            tab(4 + j * 7 - 7, k, i) = nH;
            tab(5 + j * 7 - 7, k, i) = gfgf0;
            tab(6 + j * 7 - 7, k, i) = f;
            tab(7 + j * 7 - 7, k, i) = ComTime;
        end
    end
end

TimeTab(:, 1) = reshape(tab(7, :, :), [], 1);
TimeTab(:, 2) = reshape(tab(14, :, :), [], 1);
TimeTab(:, 3) = reshape(tab(21, :, :), [], 1);

minTime = min(TimeTab')';
minTime = repmat(minTime, 1, 3);
perfratio = TimeTab ./ minTime;

taumax = max(perfratio(:));

figure(1);clf
set(0,'defaultaxesfontsize',15, ...
   'defaultaxeslinewidth',0.7, ...
   'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
set(0,'defaultlinemarkersize',10)
shapes = {'-b', '--k', '-.m', ':r', 'm', 'y'};
for i = 1 : 3
    v = sort(perfratio(:, i));
    xy = [v, ((1:length(v)) / length(v))'];
    xy = [1, 0; xy; taumax, 1];
    xynew = zeros(2 * size(xy, 1) - 1, 2);
    xynew(1:2:end, :) = xy;
    for j = 2 : 2 : size(xynew, 1)
        xynew(j, 1) = xynew(j + 1, 1);
        xynew(j, 2) = xynew(j - 1, 2);
    end

    plot(xynew(:, 1), xynew(:, 2), [shapes{i}]); hold on;
end
legend('LRBFGS', 'LRTRSR1-R', 'RTRNewton');
xlabel('\tau')
ylabel('\rho_s(\tau)');
title('Performance profile on Joint diagonalization');
savefig('StieICALRBFGSvsLRTRSR1_fig');

avetab = mean(tab, 3);

fout = fopen('StieICALRBFGSvsLRTRSR1.txt','w');
for i = 1 : size(avetab, 1)
    for j = 1 : size(avetab, 2)
        if(mod(i, 7) == 1 || mod(i, 7) == 2 || mod(i, 7) == 3 || mod(i, 7) == 4)
            fprintf(fout, '%d', round(avetab(i, j)));
        elseif(mod(i, 7) == 5 || mod(i, 7) == 6 || mod(i, 7) == 0)
            fprintf(fout, '$%s$ ', outputfloat(avetab(i, j)));
        end
        if(j < size(avetab, 2))
            fprintf(fout, ' & ', avetab(i, j));
        end
    end
    fprintf(fout,'\\\\\n');
end
fclose(fout);

function [Xopt, f, gf, gfgf0, iter, nf, ng, nR, nV, nVp, nH, ComTime, funs, grads, times, fail] = Testproblem(seed, method, prob, n, p, N)
    rng(seed);
    fail = 0;
    if(prob == 1)
        fprintf('MTestStieSoftICA seed:%d\n', seed);
%         n = 48; p = 24; N = 3000;
        Cs = randn(n, n, N);
        for i = 1 : N
            Cs(:, :, i) = (Cs(:, :, i) + Cs(:, :, i)') * 0.1 + diag(1 * ones(n, 1) + 2 * rand(n, 1)) ;%+ diag(1:(n));
%             Cs(:, :, i) = (Cs(:, :, i) + Cs(:, :, i)') * 0.1 + diag((1 : n) - n / 2);
        end
        X = orth(rand(n, p));
        SolverParams.method = method;
%         SolverParams.IsCheckParams = 1;
        SolverParams.Max_Iteration = 2000;
        SolverParams.OutputGap = 2000;
        SolverParams.LengthSY = 4;
        SolverParams.Verbose = 2;
%         SolverParams.Num_pre_funs = 10;
%         SolverParams.Accuracy = 1e-8;
%         SolverParams.Tolerance = 1e-10;
        HasHHR = 0;
        paramset = 1;
        [Xopt, f, gf, gfgf0, iter, nf, ng, nR, nV, nVp, nH, ComTime, funs, grads, times] = ...
            TestStieSoftICA(Cs, X, n, p, HasHHR, paramset, SolverParams);
    end
end

function str = outputfloat(x)
    if(x <= 0)
        sn = '-';
        x = abs(x);
    else
        sn = '';
    end
    p = log(x)/log(10);
    p = - ceil(-p);
    x = round(x * 10^(-p) * 100);
    x = x / 100;
    strx = sprintf('%3.2f', x);
    if(p ~= 0)
        str = [sn strx '_{' num2str(p) '}'];
    else
        str = [sn strx];
    end
end

function output = Afun(x, tflag, A)
    if strcmp(tflag,'notransp')
        output = A * x;
    else
        output = A' * x;
    end
end

function output = computeAH(As, Type)

    k = length(As);
    
    for i = 1:k
        invAs{i} = pinv(As{i});
    end
    
    Ar = As{1};
    for i = 2:k
       Ar = Ar + As{i};
    end
    Ar = Ar/k;
    
    if (Type == 0)
        output = Ar;
    end
    
    
    if(Type == 1)
    Ha = pinv(As{1});
    for i = 2:k
        Ha = Ha + invAs{i};
    end
    
    Ha = pinv(Ha/k);
    Arh = Ar^(0.5);
    Arinvh = Ar^(-0.5);
    temp = Arinvh * Ha * Arinvh;
    output = Arh * temp^(0.5) * Arh;
    end
end
